@@ -27,7 +27,7 @@
  * Modifications include minor bug fixes, and moving the handling of
  * endwin() into an atexit(3) function call.
  */
-#include "charva_awt_Toolkit.h"
+#include "../include/charva_awt_Toolkit.h"
 #if (defined _PDCURSES_)
     // Use the PDCurses version of the header file, which must be
     // in the current directory.
@@ -76,7 +76,13 @@ JNIEXPORT jint JNICALL Java_charva_awt_Toolkit_getAttribute
 	case 1:
 	    return (jint) A_STANDOUT;
 	case 2:
+#if (defined _PDCURSES_)
+	    // PDCurses doesn't handle UNDERLINE correctly; it
+	    // just sets the color to blue.
+	    return (jint) A_BOLD;
+#else
 	    return (jint) A_UNDERLINE;
+#endif
 	case 3:
 	    return (jint) A_REVERSE;
 	case 4:
@@ -166,7 +172,8 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_init
 
     initscr();
     keypad(stdscr, TRUE);   // enable keyboard mapping
-    noecho();       /* don't echo input */  
+    timeout(100);           // wait up to 100msec for input
+    noecho();               /* don't echo input */
     raw();
     nonl();
 
@@ -189,6 +196,8 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_init
     Java_charva_awt_Toolkit_resetClipRect(env, jo);
 }
 
+// Returns -1 if there was no character to read.
+// Otherwise returns the character that was read.
 JNIEXPORT jint JNICALL Java_charva_awt_Toolkit_readKey
   (JNIEnv *env, jobject jo)
 {
@@ -196,15 +205,17 @@ JNIEXPORT jint JNICALL Java_charva_awt_Toolkit_readKey
 
 try_again:
     c = (jint) getch();
- 
+
+ // THIS HAS ALL CHANGED: getch() is now called from the event-dispatching thread!!! (November 2004)
     // There are two known scenarios in which getch() returns ERR.
     // 1. The user resizes his PuTTY window (in which case errno == 0).
     // 2. The user closes his Telnet/SSH session (errno == EINTR).
-    if (c == ERR && errno == 0) {
-	// The window has been resized.
-	Java_charva_awt_Toolkit_resetClipRect(env, jo);
-	goto try_again;
-    }
+    //if (c == ERR && errno == 0) {
+	//    // The window has been resized.
+	//    Java_charva_awt_Toolkit_resetClipRect(env, jo);
+	//    goto try_again;
+    //}
+
     return c;
 }
 
@@ -261,14 +272,14 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_addString
 	cursorx += stringlength;
     }
     else {
-	for (i=0; i<stringlength; i++) {
-	    if (cursorx >= left && cursorx <= right)
-		my_addch(chrs[i] | attr);
-	    else {
-		cursorx++;
-		move(cursory, cursorx);
+	    for (i=0; i<stringlength; i++) {
+	        if (cursorx >= left && cursorx <= right)
+		        my_addch(chrs[i] | attr);
+	        else {
+		        cursorx++;
+		        move(cursory, cursorx);
+	        }
 	    }
-	}
     }
 
     (*env)->ReleaseStringChars(env, jstr, chrs);
@@ -292,49 +303,49 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_drawBoxNative
     int attr = 0;
 
     if (colors_started)
-	attr = COLOR_PAIR(colorpair_);
+	    attr = COLOR_PAIR(colorpair_);
 
     // If the top of the box is outside the clipping rectangle, don't bother
     // to draw the top.
     if (top_ >= top && top_ <= bottom) {
-	my_move(top_, left_);
-	my_addch_with_clip(ACS_ULCORNER | attr);    // upper left corner
+	    my_move(top_, left_);
+	    my_addch_with_clip(ACS_ULCORNER | attr);    // upper left corner
 
-	for (i=left_+1; i<right_; i++) {
-	    my_addch_with_clip(ACS_HLINE | attr);   // top horizontal line
-	}
+	    for (i=left_+1; i<right_; i++) {
+	        my_addch_with_clip(ACS_HLINE | attr);   // top horizontal line
+	    }
 
-	my_addch_with_clip(ACS_URCORNER | attr);    // upper right corner
+	    my_addch_with_clip(ACS_URCORNER | attr);    // upper right corner
     }
 
     // If the bottom of the box is outside the clipping rectangle, don't bother
     if (bottom_ >= top && bottom_ <= bottom) {
-	my_move(bottom_, left_);
-	my_addch_with_clip(ACS_LLCORNER | attr);    // lower left corner
+	    my_move(bottom_, left_);
+	    my_addch_with_clip(ACS_LLCORNER | attr);    // lower left corner
 
-	for (i=left_+1; i<right_; i++)
-	    my_addch_with_clip(ACS_HLINE | attr);   // bottom horizontal line
+	    for (i=left_+1; i<right_; i++)
+	        my_addch_with_clip(ACS_HLINE | attr);   // bottom horizontal line
 
-	my_move(bottom_, right_);
-	my_addch_with_clip(ACS_LRCORNER | attr);    // lower right corner
+	    my_move(bottom_, right_);
+	    my_addch_with_clip(ACS_LRCORNER | attr);    // lower right corner
     }
 
     // If the left side of the box is outside the clipping rectangle, don't
     // bother.
     if (left_ >= left && left_ <= right) {
-	for (i=top_+1; i<bottom_; i++) {
-	    my_move(i, left_);
-	    my_addch_with_clip(ACS_VLINE | attr);   // left vertical line
-	}
+	    for (i=top_+1; i<bottom_; i++) {
+	        my_move(i, left_);
+	        my_addch_with_clip(ACS_VLINE | attr);   // left vertical line
+	    }
     }
     //
     // If the right side of the box is outside the clipping rectangle, don't
     // bother.
     if (right_ >= left && right_ <= right) {
-	for (i=top_+1; i<bottom_; i++) {
-	    my_move(i, right_);
-	    my_addch_with_clip(ACS_VLINE | attr);   // right vertical line
-	}
+	    for (i=top_+1; i<bottom_; i++) {
+	        my_move(i, right_);
+	        my_addch_with_clip(ACS_VLINE | attr);   // right vertical line
+	    }
     }
 }
 
@@ -348,12 +359,12 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_blankBoxNative
 	attr = COLOR_PAIR(colorpair_);
 
     for (row=top_; row<=bottom_; row++) {
-	if (row < top || row > bottom)
-	    continue;	// do some clipping
+	    if (row < top || row > bottom)
+	        continue;	// do some clipping
 
-	my_move(row, left_);
-	for (col=left_; col<=right_; col++)
-	    my_addch_with_clip(' ' | attr);
+	    my_move(row, left_);
+	    for (col=left_; col<=right_; col++)
+	        my_addch_with_clip(' ' | attr);
     }
 }
 
@@ -381,9 +392,9 @@ JNIEXPORT jint JNICALL Java_charva_awt_Toolkit_getMaxColorPairs
   (JNIEnv *env, jobject jo)
 {
     if (!colors_started) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"),
-		"startColors() not called yet");
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"),
+		    "startColors() not called yet");
     }
     return (jint) COLOR_PAIRS;
 }
@@ -419,15 +430,15 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_addVerticalLine
     int attr = attrib_;
 
     if (colors_started)
-	attr |= COLOR_PAIR(colorpair_);
+	    attr |= COLOR_PAIR(colorpair_);
 
     // Set current cursor position
     x = cursorx;
     y = cursory;
 
     for (i=0; i<length_; i++) {
-	my_move(y+i, x);
-	my_addch_with_clip(ACS_VLINE | attr);
+	    my_move(y+i, x);
+	    my_addch_with_clip(ACS_VLINE | attr);
     }
 }
 
@@ -447,8 +458,8 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_addHorizontalLine
     y = cursory;
 
     for (i=0; i<length_; i++) {
-	my_move(y, x+i);
-	my_addch_with_clip(ACS_HLINE | attr);
+	    my_move(y, x+i);
+	    my_addch_with_clip(ACS_HLINE | attr);
     }
 }
 
@@ -510,11 +521,11 @@ JNIEXPORT jstring JNICALL Java_charva_awt_Toolkit_getStringCapability
     capCname = (*env)->GetStringUTFChars(env, capname_, NULL);
     capValue = tigetstr((char *) capCname);
     if (capValue == (char *) -1 || capValue == (char *) 0) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"), capCname);
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"), capCname);
     }
     else {
-	ret = (*env)->NewStringUTF(env, capValue);
+	    ret = (*env)->NewStringUTF(env, capValue);
     }
 
     (*env)->ReleaseStringUTFChars(env, capname_, capCname);
@@ -531,8 +542,8 @@ JNIEXPORT jint JNICALL Java_charva_awt_Toolkit_getNumericCapability
     capCname = (*env)->GetStringUTFChars(env, capname_, NULL);
     capValue = tigetnum((char *) capCname);
     if (capValue < 0) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"), capCname);
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"), capCname);
     }
 
     (*env)->ReleaseStringUTFChars(env, capname_, capCname);
@@ -549,11 +560,11 @@ JNIEXPORT jboolean JNICALL Java_charva_awt_Toolkit_getBooleanCapability
     capCname = (*env)->GetStringUTFChars(env, capname_, NULL);
     capValue = tigetflag((char *) capCname);
     if (capValue == -1) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"), capCname);
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"), capCname);
     }
     else {
-	ret = (capValue != 0);
+	    ret = (capValue != 0);
     }
 
     (*env)->ReleaseStringUTFChars(env, capname_, capCname);
@@ -579,7 +590,7 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_putp
 
     cstring = malloc(stringlength + 1);
     for (i=0; i<stringlength; i++) {
-	cstring[i] = (char) chrs[i];
+	    cstring[i] = (char) chrs[i];
     }
     cstring[i] = '\0';	    // terminate the C string
 
@@ -610,7 +621,7 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_print
 
     cstring = malloc(stringlength + 1);
     for (i=0; i<stringlength; i++) {
-	cstring[i] = (char) chrs[i];
+	    cstring[i] = (char) chrs[i];
     }
     cstring[i] = '\0';	    // terminate the C string
     (*env)->ReleaseStringChars(env, jstr_, chrs);
@@ -619,22 +630,21 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_print
     free(cstring);
 
     if (ret < 0) {
-	if (errno == ENODEV) {
-	    (*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"),
-		"mc4 or mc5 capability not defined");
-	}
-	else if (errno == ENOMEM) {
-	    (*env)->ThrowNew(env, (*env)->FindClass(env,
-		"java/lang/OutOfMemoryError"), "");
-	}
-	else {
-	    (*env)->ThrowNew(env, (*env)->FindClass(env,
-		"java/lang/RuntimeException"),
-		strerror(errno));
-	}
+	    if (errno == ENODEV) {
+	        (*env)->ThrowNew(env, (*env)->FindClass(env,
+		        "charva/awt/TerminfoCapabilityException"),
+		        "mc4 or mc5 capability not defined");
+	    }
+	    else if (errno == ENOMEM) {
+	        (*env)->ThrowNew(env, (*env)->FindClass(env,
+		        "java/lang/OutOfMemoryError"), "");
+	    }
+	    else {
+	        (*env)->ThrowNew(env, (*env)->FindClass(env,
+		        "java/lang/RuntimeException"),
+		        strerror(errno));
+	    }
     }
-
 }
 
 JNIEXPORT jobject JNICALL Java_charva_awt_Toolkit_getMouseEventInfo
@@ -647,20 +657,18 @@ JNIEXPORT jobject JNICALL Java_charva_awt_Toolkit_getMouseEventInfo
     jobject info;
 
     if (getmouse(&event) != OK) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"java/lang/RuntimeException"),
-		"ncurses getmouse() function returned error");
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "java/lang/RuntimeException"),
+		    "ncurses getmouse() function returned error");
     }
 
-    class_MouseEventInfo = (*env)->FindClass(env,
-	"charva/awt/MouseEventInfo");
-    id_MouseEventInfo = (*env)->GetMethodID(env,
-	class_MouseEventInfo, "<init>", "(III)V");
+    class_MouseEventInfo = (*env)->FindClass(env, "charva/awt/MouseEventInfo");
+    id_MouseEventInfo = (*env)->GetMethodID(env, class_MouseEventInfo, "<init>", "(III)V");
     button_state = event.bstate;
     x = event.x;
     y = event.y;
     info = (*env)->NewObject(env,
-	class_MouseEventInfo, id_MouseEventInfo, button_state, x, y);
+	    class_MouseEventInfo, id_MouseEventInfo, button_state, x, y);
     return info;
 }
 #endif /* ( ! defined _USE_CURSES_) */
@@ -674,9 +682,9 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_startColors
      * we initialised it.
      */
     if (!hascolors) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"),
-		"color capability not defined");
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"),
+		    "color capability not defined");
     }
     start_color();
     colors_started = 1;
@@ -689,9 +697,9 @@ JNIEXPORT void JNICALL Java_charva_awt_Toolkit_initColorPair
      * we initialised it.
      */
     if (!colors_started) {
-	(*env)->ThrowNew(env, (*env)->FindClass(env,
-		"charva/awt/TerminfoCapabilityException"),
-		"startColors() not called yet");
+	    (*env)->ThrowNew(env, (*env)->FindClass(env,
+		    "charva/awt/TerminfoCapabilityException"),
+		    "startColors() not called yet");
     }
 
     init_pair(pair_, fgnd_, bgnd_);
@@ -709,7 +717,7 @@ JNIEXPORT jstring JNICALL Java_charva_awt_Toolkit_getTtyName
 
     ttyName = ttyname(0);
     if (ttyName == NULL) {
-	ttyName = "not_a_tty";
+	    ttyName = "not_a_tty";
     }
 
     ret = (*env)->NewStringUTF(env, ttyName);
@@ -751,10 +759,11 @@ static void my_addch_with_clip(int chr_)
 {
     if (cursory >= top && cursory <= bottom && 
 		cursorx >= left && cursorx <= right) {
-	my_addch(chr_);
+
+	    my_addch(chr_);
     }
     else {
-	cursorx++;
+	    cursorx++;
 	move(cursory, cursorx);
     }
 }
